"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class PositionChangedEvent(google.protobuf.message.Message):
    """Emitted when a position changes.
    TODO: Is there a way to split this into different events without creating too much complexity?
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    TRADER_ADDRESS_FIELD_NUMBER: builtins.int
    MARGIN_FIELD_NUMBER: builtins.int
    POSITION_NOTIONAL_FIELD_NUMBER: builtins.int
    EXCHANGED_POSITION_SIZE_FIELD_NUMBER: builtins.int
    TRANSACTION_FEE_FIELD_NUMBER: builtins.int
    POSITION_SIZE_FIELD_NUMBER: builtins.int
    REALIZED_PNL_FIELD_NUMBER: builtins.int
    UNREALIZED_PNL_AFTER_FIELD_NUMBER: builtins.int
    BAD_DEBT_FIELD_NUMBER: builtins.int
    LIQUIDATION_PENALTY_FIELD_NUMBER: builtins.int
    SPOT_PRICE_FIELD_NUMBER: builtins.int
    FUNDING_PAYMENT_FIELD_NUMBER: builtins.int
    BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    BLOCK_TIME_MS_FIELD_NUMBER: builtins.int
    pair: typing.Text
    """identifier of the corresponding virtual pool for the position"""

    trader_address: typing.Text
    """owner of the position."""

    @property
    def margin(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """amount of margin backing the position."""
        pass
    position_notional: typing.Text
    """margin * leverage * vPrice. 'notional' is the virtual size times the virtual price on 'vpool'."""

    exchanged_position_size: typing.Text
    """magnitude of the change to vsize. The vsize is the amount of base assets for the position, margin * leverage * priceBasePerQuote."""

    @property
    def transaction_fee(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """transaction fee paid"""
        pass
    position_size: typing.Text
    """position virtual size after the change"""

    realized_pnl: typing.Text
    """realize profits and losses after the change"""

    unrealized_pnl_after: typing.Text
    """unrealized profits and losses after the change"""

    @property
    def bad_debt(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Amount of bad debt cleared by the PerpEF during the change. Bad debt is negative net margin past the liquidation point of a position."""
        pass
    liquidation_penalty: typing.Text
    """amt of margin (y) lost due to liquidation"""

    spot_price: typing.Text
    """vPrice defined as yRes / xRes for a vpool, where yRes is the quote reserves and xRes is the base reserves."""

    funding_payment: typing.Text
    """A funding payment made or received by the trader on the current position. 
    'fundingPayment' is positive if 'owner' is the sender and negative if 'owner'
    is the receiver of the payment. Its magnitude is abs(vSize * fundingRate). 
    Funding payments act to converge the mark price (vPrice) and index price 
    (average price on major exchanges).
    """

    block_height: builtins.int
    """The block number at which this position was changed."""

    block_time_ms: builtins.int
    """The block time in unix milliseconds at which this position was changed."""

    def __init__(self,
        *,
        pair: typing.Text = ...,
        trader_address: typing.Text = ...,
        margin: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        position_notional: typing.Text = ...,
        exchanged_position_size: typing.Text = ...,
        transaction_fee: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        position_size: typing.Text = ...,
        realized_pnl: typing.Text = ...,
        unrealized_pnl_after: typing.Text = ...,
        bad_debt: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        liquidation_penalty: typing.Text = ...,
        spot_price: typing.Text = ...,
        funding_payment: typing.Text = ...,
        block_height: builtins.int = ...,
        block_time_ms: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bad_debt",b"bad_debt","margin",b"margin","transaction_fee",b"transaction_fee"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bad_debt",b"bad_debt","block_height",b"block_height","block_time_ms",b"block_time_ms","exchanged_position_size",b"exchanged_position_size","funding_payment",b"funding_payment","liquidation_penalty",b"liquidation_penalty","margin",b"margin","pair",b"pair","position_notional",b"position_notional","position_size",b"position_size","realized_pnl",b"realized_pnl","spot_price",b"spot_price","trader_address",b"trader_address","transaction_fee",b"transaction_fee","unrealized_pnl_after",b"unrealized_pnl_after"]) -> None: ...
global___PositionChangedEvent = PositionChangedEvent

class PositionLiquidatedEvent(google.protobuf.message.Message):
    """Emitted when a position is liquidated."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    TRADER_ADDRESS_FIELD_NUMBER: builtins.int
    EXCHANGED_QUOTE_AMOUNT_FIELD_NUMBER: builtins.int
    EXCHANGED_POSITION_SIZE_FIELD_NUMBER: builtins.int
    LIQUIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    FEE_TO_LIQUIDATOR_FIELD_NUMBER: builtins.int
    FEE_TO_ECOSYSTEM_FUND_FIELD_NUMBER: builtins.int
    BAD_DEBT_FIELD_NUMBER: builtins.int
    MARGIN_FIELD_NUMBER: builtins.int
    POSITION_NOTIONAL_FIELD_NUMBER: builtins.int
    POSITION_SIZE_FIELD_NUMBER: builtins.int
    UNREALIZEDPNL_FIELD_NUMBER: builtins.int
    MARK_PRICE_FIELD_NUMBER: builtins.int
    BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    BLOCK_TIME_MS_FIELD_NUMBER: builtins.int
    pair: typing.Text
    """identifier of the corresponding virtual pool for the position"""

    trader_address: typing.Text
    """owner of the position."""

    exchanged_quote_amount: typing.Text
    """margin * leverage * vPrice. 'notional' is the virtual size times  the virtual price on 'vpool'."""

    exchanged_position_size: typing.Text
    """virtual amount of base assets for the position, which would be margin * leverage * priceBasePerQuote."""

    liquidator_address: typing.Text
    """Address of the account that executed the tx."""

    @property
    def fee_to_liquidator(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Commission (in margin units) received by 'liquidator'."""
        pass
    @property
    def fee_to_ecosystem_fund(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Commission (in margin units) given to the ecosystem fund."""
        pass
    @property
    def bad_debt(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """ Bad debt (margin units) cleared by the PerpEF during the tx. Bad debt is negative net margin past the liquidation point of a position."""
        pass
    @property
    def margin(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Remaining margin in the position after liquidation"""
        pass
    position_notional: typing.Text
    """Remaining position notional in the position after liquidation"""

    position_size: typing.Text
    """Remaining position size in the position after liquidation"""

    unrealizedPnl: typing.Text
    """Unrealized PnL in the position after liquidation"""

    mark_price: typing.Text
    """Spot price of the vAMM after liquidation"""

    block_height: builtins.int
    """The block number at which this liquidation occured."""

    block_time_ms: builtins.int
    """The unix timestamp in milliseconds at which this liquidation occured."""

    def __init__(self,
        *,
        pair: typing.Text = ...,
        trader_address: typing.Text = ...,
        exchanged_quote_amount: typing.Text = ...,
        exchanged_position_size: typing.Text = ...,
        liquidator_address: typing.Text = ...,
        fee_to_liquidator: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        fee_to_ecosystem_fund: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        bad_debt: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        margin: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        position_notional: typing.Text = ...,
        position_size: typing.Text = ...,
        unrealizedPnl: typing.Text = ...,
        mark_price: typing.Text = ...,
        block_height: builtins.int = ...,
        block_time_ms: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bad_debt",b"bad_debt","fee_to_ecosystem_fund",b"fee_to_ecosystem_fund","fee_to_liquidator",b"fee_to_liquidator","margin",b"margin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bad_debt",b"bad_debt","block_height",b"block_height","block_time_ms",b"block_time_ms","exchanged_position_size",b"exchanged_position_size","exchanged_quote_amount",b"exchanged_quote_amount","fee_to_ecosystem_fund",b"fee_to_ecosystem_fund","fee_to_liquidator",b"fee_to_liquidator","liquidator_address",b"liquidator_address","margin",b"margin","mark_price",b"mark_price","pair",b"pair","position_notional",b"position_notional","position_size",b"position_size","trader_address",b"trader_address","unrealizedPnl",b"unrealizedPnl"]) -> None: ...
global___PositionLiquidatedEvent = PositionLiquidatedEvent

class PositionSettledEvent(google.protobuf.message.Message):
    """Emitted when a position is settled."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    TRADER_ADDRESS_FIELD_NUMBER: builtins.int
    SETTLED_COINS_FIELD_NUMBER: builtins.int
    pair: typing.Text
    """Identifier for the virtual pool of the position."""

    trader_address: typing.Text
    """Owner of the position."""

    @property
    def settled_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """Settled coin as dictated by the settlement price of the vpool."""
        pass
    def __init__(self,
        *,
        pair: typing.Text = ...,
        trader_address: typing.Text = ...,
        settled_coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair",b"pair","settled_coins",b"settled_coins","trader_address",b"trader_address"]) -> None: ...
global___PositionSettledEvent = PositionSettledEvent

class FundingRateChangedEvent(google.protobuf.message.Message):
    """Emitted when a new funding rate is calculated."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    MARK_PRICE_FIELD_NUMBER: builtins.int
    INDEX_PRICE_FIELD_NUMBER: builtins.int
    LATEST_FUNDING_RATE_FIELD_NUMBER: builtins.int
    CUMULATIVE_FUNDING_RATE_FIELD_NUMBER: builtins.int
    BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    BLOCK_TIME_MS_FIELD_NUMBER: builtins.int
    pair: typing.Text
    """The pair for which the funding rate was calculated."""

    mark_price: typing.Text
    """The mark price of the pair."""

    index_price: typing.Text
    """The oracle index price of the pair."""

    latest_funding_rate: typing.Text
    """The latest funding rate."""

    cumulative_funding_rate: typing.Text
    """The latest cumulative funding rate.
    The funding rate a position will pay is the difference between this value
    and the latest cumulative funding rate on the position.
    """

    block_height: builtins.int
    """The block number at which the funding rate was calculated."""

    block_time_ms: builtins.int
    """The block time in unix milliseconds at which the funding rate was calculated."""

    def __init__(self,
        *,
        pair: typing.Text = ...,
        mark_price: typing.Text = ...,
        index_price: typing.Text = ...,
        latest_funding_rate: typing.Text = ...,
        cumulative_funding_rate: typing.Text = ...,
        block_height: builtins.int = ...,
        block_time_ms: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_height",b"block_height","block_time_ms",b"block_time_ms","cumulative_funding_rate",b"cumulative_funding_rate","index_price",b"index_price","latest_funding_rate",b"latest_funding_rate","mark_price",b"mark_price","pair",b"pair"]) -> None: ...
global___FundingRateChangedEvent = FundingRateChangedEvent



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nibiru',
 'nibiru.clients',
 'nibiru.composers',
 'nibiru.proto',
 'nibiru.proto.common',
 'nibiru.proto.confio',
 'nibiru.proto.cosmos.auth.v1beta1',
 'nibiru.proto.cosmos.authz.v1beta1',
 'nibiru.proto.cosmos.bank.v1beta1',
 'nibiru.proto.cosmos.base.abci.v1beta1',
 'nibiru.proto.cosmos.base.kv.v1beta1',
 'nibiru.proto.cosmos.base.query.v1beta1',
 'nibiru.proto.cosmos.base.reflection.v1beta1',
 'nibiru.proto.cosmos.base.reflection.v2alpha1',
 'nibiru.proto.cosmos.base.snapshots.v1beta1',
 'nibiru.proto.cosmos.base.store.v1beta1',
 'nibiru.proto.cosmos.base.tendermint.v1beta1',
 'nibiru.proto.cosmos.base.v1beta1',
 'nibiru.proto.cosmos.capability.v1beta1',
 'nibiru.proto.cosmos.crisis.v1beta1',
 'nibiru.proto.cosmos.crypto.ed25519',
 'nibiru.proto.cosmos.crypto.multisig',
 'nibiru.proto.cosmos.crypto.multisig.v1beta1',
 'nibiru.proto.cosmos.crypto.secp256k1',
 'nibiru.proto.cosmos.crypto.secp256r1',
 'nibiru.proto.cosmos.distribution.v1beta1',
 'nibiru.proto.cosmos.evidence.v1beta1',
 'nibiru.proto.cosmos.feegrant.v1beta1',
 'nibiru.proto.cosmos.genutil.v1beta1',
 'nibiru.proto.cosmos.gov.v1beta1',
 'nibiru.proto.cosmos.mint.v1beta1',
 'nibiru.proto.cosmos.params.v1beta1',
 'nibiru.proto.cosmos.slashing.v1beta1',
 'nibiru.proto.cosmos.staking.v1beta1',
 'nibiru.proto.cosmos.tx.signing.v1beta1',
 'nibiru.proto.cosmos.tx.v1beta1',
 'nibiru.proto.cosmos.upgrade.v1beta1',
 'nibiru.proto.cosmos.vesting.v1beta1',
 'nibiru.proto.cosmos_proto',
 'nibiru.proto.dex.v1',
 'nibiru.proto.epochs',
 'nibiru.proto.gogoproto',
 'nibiru.proto.google.api',
 'nibiru.proto.google.protobuf',
 'nibiru.proto.incentivization.v1',
 'nibiru.proto.lockup.v1',
 'nibiru.proto.oracle.v1beta1',
 'nibiru.proto.perp.v1',
 'nibiru.proto.pricefeed',
 'nibiru.proto.stablecoin',
 'nibiru.proto.tendermint.abci',
 'nibiru.proto.tendermint.crypto',
 'nibiru.proto.tendermint.libs.bits',
 'nibiru.proto.tendermint.p2p',
 'nibiru.proto.tendermint.types',
 'nibiru.proto.tendermint.version',
 'nibiru.proto.vpool.v1',
 'nibiru.sdks',
 'nibiru.sdks.tx']

package_data = \
{'': ['*']}

install_requires = \
['aiocron>=1.8,<2.0',
 'aiohttp>=3.8.1,<4.0.0',
 'asyncio>=3.4.3,<4.0.0',
 'bech32>=1.2.0,<2.0.0',
 'bip32>=3.1,<4.0',
 'coincurve>=17.0.0,<18.0.0',
 'ecdsa>=0.18.0,<0.19.0',
 'eip712-structs>=1.1.0,<2.0.0',
 'grpcio>=1.48.0,<2.0.0',
 'hdwallets>=0.1.2,<0.2.0',
 'mnemonic>=0.20,<0.21',
 'protobuf==3.20.1',
 'pysha3>=1.0.2,<2.0.0',
 'pytest>=7.1.2,<8.0.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'nibiru',
    'version': '0.0.14a1',
    'description': 'Python SDK for interacting with Nibiru.',
    'long_description': None,
    'author': 'Nibiru Chain',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from oracle.v1beta1 import tx_pb2 as oracle_dot_v1beta1_dot_tx__pb2


class MsgStub(object):
    """Msg defines the oracle Msg service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AggregateExchangeRatePrevote = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Msg/AggregateExchangeRatePrevote',
                request_serializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRatePrevote.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRatePrevoteResponse.FromString,
                )
        self.AggregateExchangeRateVote = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Msg/AggregateExchangeRateVote',
                request_serializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRateVote.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRateVoteResponse.FromString,
                )
        self.DelegateFeedConsent = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Msg/DelegateFeedConsent',
                request_serializer=oracle_dot_v1beta1_dot_tx__pb2.MsgDelegateFeedConsent.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_tx__pb2.MsgDelegateFeedConsentResponse.FromString,
                )


class MsgServicer(object):
    """Msg defines the oracle Msg service.
    """

    def AggregateExchangeRatePrevote(self, request, context):
        """AggregateExchangeRatePrevote defines a method for submitting
        aggregate exchange rate prevote
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AggregateExchangeRateVote(self, request, context):
        """AggregateExchangeRateVote defines a method for submitting
        aggregate exchange rate vote
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DelegateFeedConsent(self, request, context):
        """DelegateFeedConsent defines a method for setting the feeder delegation
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MsgServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AggregateExchangeRatePrevote': grpc.unary_unary_rpc_method_handler(
                    servicer.AggregateExchangeRatePrevote,
                    request_deserializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRatePrevote.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRatePrevoteResponse.SerializeToString,
            ),
            'AggregateExchangeRateVote': grpc.unary_unary_rpc_method_handler(
                    servicer.AggregateExchangeRateVote,
                    request_deserializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRateVote.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRateVoteResponse.SerializeToString,
            ),
            'DelegateFeedConsent': grpc.unary_unary_rpc_method_handler(
                    servicer.DelegateFeedConsent,
                    request_deserializer=oracle_dot_v1beta1_dot_tx__pb2.MsgDelegateFeedConsent.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_tx__pb2.MsgDelegateFeedConsentResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nibiru.oracle.v1beta1.Msg', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Msg(object):
    """Msg defines the oracle Msg service.
    """

    @staticmethod
    def AggregateExchangeRatePrevote(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Msg/AggregateExchangeRatePrevote',
            oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRatePrevote.SerializeToString,
            oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRatePrevoteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AggregateExchangeRateVote(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Msg/AggregateExchangeRateVote',
            oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRateVote.SerializeToString,
            oracle_dot_v1beta1_dot_tx__pb2.MsgAggregateExchangeRateVoteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DelegateFeedConsent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Msg/DelegateFeedConsent',
            oracle_dot_v1beta1_dot_tx__pb2.MsgDelegateFeedConsent.SerializeToString,
            oracle_dot_v1beta1_dot_tx__pb2.MsgDelegateFeedConsentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import common.common_pb2
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Side:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _SideEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Side.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SIDE_UNSPECIFIED: _Side.ValueType  # 0
    BUY: _Side.ValueType  # 1
    SELL: _Side.ValueType  # 2
class Side(_Side, metaclass=_SideEnumTypeWrapper):
    pass

SIDE_UNSPECIFIED: Side.ValueType  # 0
BUY: Side.ValueType  # 1
SELL: Side.ValueType  # 2
global___Side = Side


class _PnLCalcOption:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _PnLCalcOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PnLCalcOption.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    PNL_CALC_OPTION_UNSPECIFIED: _PnLCalcOption.ValueType  # 0
    SPOT_PRICE: _PnLCalcOption.ValueType  # 1
    TWAP: _PnLCalcOption.ValueType  # 2
    ORACLE: _PnLCalcOption.ValueType  # 3
class PnLCalcOption(_PnLCalcOption, metaclass=_PnLCalcOptionEnumTypeWrapper):
    pass

PNL_CALC_OPTION_UNSPECIFIED: PnLCalcOption.ValueType  # 0
SPOT_PRICE: PnLCalcOption.ValueType  # 1
TWAP: PnLCalcOption.ValueType  # 2
ORACLE: PnLCalcOption.ValueType  # 3
global___PnLCalcOption = PnLCalcOption


class _PnLPreferenceOption:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _PnLPreferenceOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PnLPreferenceOption.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    PNL_PREFERENCE_UNSPECIFIED: _PnLPreferenceOption.ValueType  # 0
    MAX: _PnLPreferenceOption.ValueType  # 1
    MIN: _PnLPreferenceOption.ValueType  # 2
class PnLPreferenceOption(_PnLPreferenceOption, metaclass=_PnLPreferenceOptionEnumTypeWrapper):
    pass

PNL_PREFERENCE_UNSPECIFIED: PnLPreferenceOption.ValueType  # 0
MAX: PnLPreferenceOption.ValueType  # 1
MIN: PnLPreferenceOption.ValueType  # 2
global___PnLPreferenceOption = PnLPreferenceOption


class _MarginCalculationPriceOption:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _MarginCalculationPriceOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MarginCalculationPriceOption.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    MARGIN_CALCULATION_PRICE_OPTION_UNSPECIFIED: _MarginCalculationPriceOption.ValueType  # 0
    SPOT: _MarginCalculationPriceOption.ValueType  # 1
    INDEX: _MarginCalculationPriceOption.ValueType  # 2
    MAX_PNL: _MarginCalculationPriceOption.ValueType  # 3
class MarginCalculationPriceOption(_MarginCalculationPriceOption, metaclass=_MarginCalculationPriceOptionEnumTypeWrapper):
    pass

MARGIN_CALCULATION_PRICE_OPTION_UNSPECIFIED: MarginCalculationPriceOption.ValueType  # 0
SPOT: MarginCalculationPriceOption.ValueType  # 1
INDEX: MarginCalculationPriceOption.ValueType  # 2
MAX_PNL: MarginCalculationPriceOption.ValueType  # 3
global___MarginCalculationPriceOption = MarginCalculationPriceOption


class Params(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STOPPED_FIELD_NUMBER: builtins.int
    FEE_POOL_FEE_RATIO_FIELD_NUMBER: builtins.int
    ECOSYSTEM_FUND_FEE_RATIO_FIELD_NUMBER: builtins.int
    LIQUIDATION_FEE_RATIO_FIELD_NUMBER: builtins.int
    PARTIAL_LIQUIDATION_RATIO_FIELD_NUMBER: builtins.int
    FUNDING_RATE_INTERVAL_FIELD_NUMBER: builtins.int
    TWAP_LOOKBACK_WINDOW_FIELD_NUMBER: builtins.int
    stopped: builtins.bool
    """stopped identifies if the perp exchange is stopped or not"""

    fee_pool_fee_ratio: typing.Text
    """FeePoolFeeRatio is the ratio transferred to the the fee pool"""

    ecosystem_fund_fee_ratio: typing.Text
    """EcosystemFundFeeRatio is the ratio transferred to the PerpEF."""

    liquidation_fee_ratio: typing.Text
    """LiquidationFeeRatio is the percentage of liquidated position that will be given
    to out as a reward. Half of the liquidation fee is given to the liquidator,
    and the other half is given to the ecosystem fund.
    """

    partial_liquidation_ratio: typing.Text
    """PartialLiquidationRatio is the share we try to liquidate if the margin is higher than liquidation fee"""

    funding_rate_interval: typing.Text
    """specifies the interval on which the funding rate is updated"""

    @property
    def twap_lookback_window(self) -> google.protobuf.duration_pb2.Duration:
        """amount of time to look back for TWAP calculations"""
        pass
    def __init__(self,
        *,
        stopped: builtins.bool = ...,
        fee_pool_fee_ratio: typing.Text = ...,
        ecosystem_fund_fee_ratio: typing.Text = ...,
        liquidation_fee_ratio: typing.Text = ...,
        partial_liquidation_ratio: typing.Text = ...,
        funding_rate_interval: typing.Text = ...,
        twap_lookback_window: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["twap_lookback_window",b"twap_lookback_window"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ecosystem_fund_fee_ratio",b"ecosystem_fund_fee_ratio","fee_pool_fee_ratio",b"fee_pool_fee_ratio","funding_rate_interval",b"funding_rate_interval","liquidation_fee_ratio",b"liquidation_fee_ratio","partial_liquidation_ratio",b"partial_liquidation_ratio","stopped",b"stopped","twap_lookback_window",b"twap_lookback_window"]) -> None: ...
global___Params = Params

class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the perp module's genesis state."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    VAULT_BALANCE_FIELD_NUMBER: builtins.int
    PERP_EF_BALANCE_FIELD_NUMBER: builtins.int
    FEE_POOL_BALANCE_FIELD_NUMBER: builtins.int
    PAIR_METADATA_FIELD_NUMBER: builtins.int
    POSITIONS_FIELD_NUMBER: builtins.int
    PREPAID_BAD_DEBTS_FIELD_NUMBER: builtins.int
    WHITELISTED_ADDRESSES_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> global___Params: ...
    @property
    def vault_balance(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def perp_ef_balance(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def fee_pool_balance(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def pair_metadata(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PairMetadata]: ...
    @property
    def positions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Position]: ...
    @property
    def prepaid_bad_debts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PrepaidBadDebt]: ...
    @property
    def whitelisted_addresses(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    def __init__(self,
        *,
        params: typing.Optional[global___Params] = ...,
        vault_balance: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        perp_ef_balance: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        fee_pool_balance: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        pair_metadata: typing.Optional[typing.Iterable[global___PairMetadata]] = ...,
        positions: typing.Optional[typing.Iterable[global___Position]] = ...,
        prepaid_bad_debts: typing.Optional[typing.Iterable[global___PrepaidBadDebt]] = ...,
        whitelisted_addresses: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fee_pool_balance",b"fee_pool_balance","pair_metadata",b"pair_metadata","params",b"params","perp_ef_balance",b"perp_ef_balance","positions",b"positions","prepaid_bad_debts",b"prepaid_bad_debts","vault_balance",b"vault_balance","whitelisted_addresses",b"whitelisted_addresses"]) -> None: ...
global___GenesisState = GenesisState

class Position(google.protobuf.message.Message):
    """Position identifies and records information on a user's position on one of
    the virtual liquidity pools.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TRADER_ADDRESS_FIELD_NUMBER: builtins.int
    PAIR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    MARGIN_FIELD_NUMBER: builtins.int
    OPEN_NOTIONAL_FIELD_NUMBER: builtins.int
    LAST_UPDATE_CUMULATIVE_PREMIUM_FRACTION_FIELD_NUMBER: builtins.int
    BLOCK_NUMBER_FIELD_NUMBER: builtins.int
    trader_address: typing.Text
    """address identifies the address owner of this position"""

    @property
    def pair(self) -> common.common_pb2.AssetPair:
        """pair identifies the pair associated with this position"""
        pass
    size: typing.Text
    """Position size.
    signed int
    """

    margin: typing.Text
    """Amount of margin remaining in the position."""

    open_notional: typing.Text
    """OpenNotional is the quote denom value of the position when opening.
    Used to calculate PnL.
    """

    last_update_cumulative_premium_fraction: typing.Text
    """The last cumulative funding payment this position has applied.
    Used to calculate the next funding payment.
    int
    """

    block_number: builtins.int
    """BlockNumber is the last block number when this position changed."""

    def __init__(self,
        *,
        trader_address: typing.Text = ...,
        pair: typing.Optional[common.common_pb2.AssetPair] = ...,
        size: typing.Text = ...,
        margin: typing.Text = ...,
        open_notional: typing.Text = ...,
        last_update_cumulative_premium_fraction: typing.Text = ...,
        block_number: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pair",b"pair"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_number",b"block_number","last_update_cumulative_premium_fraction",b"last_update_cumulative_premium_fraction","margin",b"margin","open_notional",b"open_notional","pair",b"pair","size",b"size","trader_address",b"trader_address"]) -> None: ...
global___Position = Position

class PositionResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POSITION_FIELD_NUMBER: builtins.int
    EXCHANGED_NOTIONAL_VALUE_FIELD_NUMBER: builtins.int
    EXCHANGED_POSITION_SIZE_FIELD_NUMBER: builtins.int
    BAD_DEBT_FIELD_NUMBER: builtins.int
    FUNDING_PAYMENT_FIELD_NUMBER: builtins.int
    REALIZED_PNL_FIELD_NUMBER: builtins.int
    UNREALIZED_PNL_AFTER_FIELD_NUMBER: builtins.int
    MARGIN_TO_VAULT_FIELD_NUMBER: builtins.int
    POSITION_NOTIONAL_FIELD_NUMBER: builtins.int
    @property
    def position(self) -> global___Position: ...
    exchanged_notional_value: typing.Text
    """The amount of quote assets exchanged."""

    exchanged_position_size: typing.Text
    """The amount of base assets exchanged."""

    bad_debt: typing.Text
    """The amount of bad debt accrued during this position change.
    Measured in absolute value of quote units.
    If greater than zero, then the position change event will likely fail.
    """

    funding_payment: typing.Text
    """The funding payment applied on this position change."""

    realized_pnl: typing.Text
    """The amount of PnL realized on this position changed, measured in quote units."""

    unrealized_pnl_after: typing.Text
    """The unrealized PnL in the position after the position change."""

    margin_to_vault: typing.Text
    """The amount of margin the trader has to give to the vault.
    A negative value means the vault pays the trader.
    """

    position_notional: typing.Text
    """The position's notional value after the position change, measured in quote units."""

    def __init__(self,
        *,
        position: typing.Optional[global___Position] = ...,
        exchanged_notional_value: typing.Text = ...,
        exchanged_position_size: typing.Text = ...,
        bad_debt: typing.Text = ...,
        funding_payment: typing.Text = ...,
        realized_pnl: typing.Text = ...,
        unrealized_pnl_after: typing.Text = ...,
        margin_to_vault: typing.Text = ...,
        position_notional: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["position",b"position"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bad_debt",b"bad_debt","exchanged_notional_value",b"exchanged_notional_value","exchanged_position_size",b"exchanged_position_size","funding_payment",b"funding_payment","margin_to_vault",b"margin_to_vault","position",b"position","position_notional",b"position_notional","realized_pnl",b"realized_pnl","unrealized_pnl_after",b"unrealized_pnl_after"]) -> None: ...
global___PositionResp = PositionResp

class LiquidateResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BAD_DEBT_FIELD_NUMBER: builtins.int
    FEE_TO_LIQUIDATOR_FIELD_NUMBER: builtins.int
    FEE_TO_PERP_ECOSYSTEM_FUND_FIELD_NUMBER: builtins.int
    LIQUIDATOR_FIELD_NUMBER: builtins.int
    POSITION_RESP_FIELD_NUMBER: builtins.int
    bad_debt: typing.Text
    """Amount of bad debt created by the liquidation event"""

    fee_to_liquidator: typing.Text
    """Fee paid to the liquidator"""

    fee_to_perp_ecosystem_fund: typing.Text
    """Fee paid to the Perp EF fund"""

    liquidator: typing.Text
    """Address of the liquidator"""

    @property
    def position_resp(self) -> global___PositionResp:
        """Position response from the close or open reverse position"""
        pass
    def __init__(self,
        *,
        bad_debt: typing.Text = ...,
        fee_to_liquidator: typing.Text = ...,
        fee_to_perp_ecosystem_fund: typing.Text = ...,
        liquidator: typing.Text = ...,
        position_resp: typing.Optional[global___PositionResp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["position_resp",b"position_resp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bad_debt",b"bad_debt","fee_to_liquidator",b"fee_to_liquidator","fee_to_perp_ecosystem_fund",b"fee_to_perp_ecosystem_fund","liquidator",b"liquidator","position_resp",b"position_resp"]) -> None: ...
global___LiquidateResp = LiquidateResp

class PairMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    CUMULATIVE_PREMIUM_FRACTIONS_FIELD_NUMBER: builtins.int
    @property
    def pair(self) -> common.common_pb2.AssetPair: ...
    @property
    def cumulative_premium_fractions(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    def __init__(self,
        *,
        pair: typing.Optional[common.common_pb2.AssetPair] = ...,
        cumulative_premium_fractions: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pair",b"pair"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cumulative_premium_fractions",b"cumulative_premium_fractions","pair",b"pair"]) -> None: ...
global___PairMetadata = PairMetadata

class PrepaidBadDebt(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DENOM_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    denom: typing.Text
    amount: typing.Text
    def __init__(self,
        *,
        denom: typing.Text = ...,
        amount: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","denom",b"denom"]) -> None: ...
global___PrepaidBadDebt = PrepaidBadDebt



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.base.v1beta1.coin_pb2
import dex.v1.params_pb2
import dex.v1.pool_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is request type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> dex.v1.params_pb2.Params:
        """params holds all the parameters of this module."""
        pass
    def __init__(self,
        *,
        params: typing.Optional[dex.v1.params_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse

class QueryPoolNumberRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryPoolNumberRequest = QueryPoolNumberRequest

class QueryPoolNumberResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryPoolNumberResponse = QueryPoolNumberResponse

class QueryPoolRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryPoolRequest = QueryPoolRequest

class QueryPoolResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_FIELD_NUMBER: builtins.int
    @property
    def pool(self) -> dex.v1.pool_pb2.Pool: ...
    def __init__(self,
        *,
        pool: typing.Optional[dex.v1.pool_pb2.Pool] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool",b"pool"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool",b"pool"]) -> None: ...
global___QueryPoolResponse = QueryPoolResponse

class QueryPoolsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> None: ...
global___QueryPoolsRequest = QueryPoolsRequest

class QueryPoolsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOLS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def pools(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[dex.v1.pool_pb2.Pool]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        pools: typing.Optional[typing.Iterable[dex.v1.pool_pb2.Pool]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","pools",b"pools"]) -> None: ...
global___QueryPoolsResponse = QueryPoolsResponse

class QueryPoolParamsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryPoolParamsRequest = QueryPoolParamsRequest

class QueryPoolParamsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_PARAMS_FIELD_NUMBER: builtins.int
    @property
    def pool_params(self) -> dex.v1.pool_pb2.PoolParams: ...
    def __init__(self,
        *,
        pool_params: typing.Optional[dex.v1.pool_pb2.PoolParams] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_params",b"pool_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_params",b"pool_params"]) -> None: ...
global___QueryPoolParamsResponse = QueryPoolParamsResponse

class QueryNumPoolsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryNumPoolsRequest = QueryNumPoolsRequest

class QueryNumPoolsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NUM_POOLS_FIELD_NUMBER: builtins.int
    num_pools: builtins.int
    def __init__(self,
        *,
        num_pools: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["num_pools",b"num_pools"]) -> None: ...
global___QueryNumPoolsResponse = QueryNumPoolsResponse

class QueryTotalLiquidityRequest(google.protobuf.message.Message):
    """--------------------------------------------
    Query total liquidity the protocol
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryTotalLiquidityRequest = QueryTotalLiquidityRequest

class QueryTotalLiquidityResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LIQUIDITY_FIELD_NUMBER: builtins.int
    @property
    def liquidity(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        liquidity: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["liquidity",b"liquidity"]) -> None: ...
global___QueryTotalLiquidityResponse = QueryTotalLiquidityResponse

class QueryTotalPoolLiquidityRequest(google.protobuf.message.Message):
    """--------------------------------------------
    Query total liquidity for a pool
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryTotalPoolLiquidityRequest = QueryTotalPoolLiquidityRequest

class QueryTotalPoolLiquidityResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LIQUIDITY_FIELD_NUMBER: builtins.int
    @property
    def liquidity(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        liquidity: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["liquidity",b"liquidity"]) -> None: ...
global___QueryTotalPoolLiquidityResponse = QueryTotalPoolLiquidityResponse

class QueryTotalSharesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryTotalSharesRequest = QueryTotalSharesRequest

class QueryTotalSharesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOTAL_SHARES_FIELD_NUMBER: builtins.int
    @property
    def total_shares(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """sum of all LP tokens sent out"""
        pass
    def __init__(self,
        *,
        total_shares: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["total_shares",b"total_shares"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["total_shares",b"total_shares"]) -> None: ...
global___QueryTotalSharesResponse = QueryTotalSharesResponse

class QuerySpotPriceRequest(google.protobuf.message.Message):
    """Returns the amount of tokenInDenom to produce 1 tokenOutDenom
    For example, if the price of NIBI = 9.123 NUSD, then setting tokenInDenom=NUSD
    and tokenOutDenom=NIBI would give "9.123".
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKEN_IN_DENOM_FIELD_NUMBER: builtins.int
    TOKEN_OUT_DENOM_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    token_in_denom: typing.Text
    """the denomination of the token you are giving into the pool"""

    token_out_denom: typing.Text
    """the denomination of the token you are taking out of the pool"""

    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        token_in_denom: typing.Text = ...,
        token_out_denom: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","token_in_denom",b"token_in_denom","token_out_denom",b"token_out_denom"]) -> None: ...
global___QuerySpotPriceRequest = QuerySpotPriceRequest

class QuerySpotPriceResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SPOT_PRICE_FIELD_NUMBER: builtins.int
    spot_price: typing.Text
    def __init__(self,
        *,
        spot_price: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["spot_price",b"spot_price"]) -> None: ...
global___QuerySpotPriceResponse = QuerySpotPriceResponse

class QuerySwapExactAmountInRequest(google.protobuf.message.Message):
    """Given an exact amount of tokens in and a target tokenOutDenom, calculates
    the expected amount of tokens out received from a swap.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKEN_IN_FIELD_NUMBER: builtins.int
    TOKEN_OUT_DENOM_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    @property
    def token_in(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    token_out_denom: typing.Text
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        token_in: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        token_out_denom: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_in",b"token_in"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","token_in",b"token_in","token_out_denom",b"token_out_denom"]) -> None: ...
global___QuerySwapExactAmountInRequest = QuerySwapExactAmountInRequest

class QuerySwapExactAmountInResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKEN_OUT_FIELD_NUMBER: builtins.int
    @property
    def token_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        token_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_out",b"token_out"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token_out",b"token_out"]) -> None: ...
global___QuerySwapExactAmountInResponse = QuerySwapExactAmountInResponse

class QuerySwapExactAmountOutRequest(google.protobuf.message.Message):
    """Given an exact amount of tokens out and a target tokenInDenom, calculates
    the expected amount of tokens in required to do the swap.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKEN_OUT_FIELD_NUMBER: builtins.int
    TOKEN_IN_DENOM_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    @property
    def token_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    token_in_denom: typing.Text
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        token_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        token_in_denom: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_out",b"token_out"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","token_in_denom",b"token_in_denom","token_out",b"token_out"]) -> None: ...
global___QuerySwapExactAmountOutRequest = QuerySwapExactAmountOutRequest

class QuerySwapExactAmountOutResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKEN_IN_FIELD_NUMBER: builtins.int
    @property
    def token_in(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        token_in: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_in",b"token_in"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token_in",b"token_in"]) -> None: ...
global___QuerySwapExactAmountOutResponse = QuerySwapExactAmountOutResponse

class QueryJoinExactAmountInRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKENS_IN_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    @property
    def tokens_in(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        tokens_in: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","tokens_in",b"tokens_in"]) -> None: ...
global___QueryJoinExactAmountInRequest = QueryJoinExactAmountInRequest

class QueryJoinExactAmountInResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_SHARES_OUT_FIELD_NUMBER: builtins.int
    REM_COINS_FIELD_NUMBER: builtins.int
    pool_shares_out: typing.Text
    """amount of pool shares returned to user after join"""

    @property
    def rem_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """coins remaining after pool join"""
        pass
    def __init__(self,
        *,
        pool_shares_out: typing.Text = ...,
        rem_coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_shares_out",b"pool_shares_out","rem_coins",b"rem_coins"]) -> None: ...
global___QueryJoinExactAmountInResponse = QueryJoinExactAmountInResponse

class QueryJoinExactAmountOutRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryJoinExactAmountOutRequest = QueryJoinExactAmountOutRequest

class QueryJoinExactAmountOutResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryJoinExactAmountOutResponse = QueryJoinExactAmountOutResponse

class QueryExitExactAmountInRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    POOL_SHARES_IN_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    pool_shares_in: typing.Text
    """amount of pool shares to return to pool"""

    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        pool_shares_in: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","pool_shares_in",b"pool_shares_in"]) -> None: ...
global___QueryExitExactAmountInRequest = QueryExitExactAmountInRequest

class QueryExitExactAmountInResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKENS_OUT_FIELD_NUMBER: builtins.int
    @property
    def tokens_out(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """coins obtained after exiting"""
        pass
    def __init__(self,
        *,
        tokens_out: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tokens_out",b"tokens_out"]) -> None: ...
global___QueryExitExactAmountInResponse = QueryExitExactAmountInResponse

class QueryExitExactAmountOutRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___QueryExitExactAmountOutRequest = QueryExitExactAmountOutRequest

class QueryExitExactAmountOutResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryExitExactAmountOutResponse = QueryExitExactAmountOutResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.api.http_pb2
import google.protobuf.descriptor
import google.protobuf.descriptor_pb2
import google.protobuf.internal.extension_dict

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

HTTP_FIELD_NUMBER: builtins.int
http: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MethodOptions, google.api.http_pb2.HttpRule]
"""See `HttpRule`."""



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import common.common_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Params(google.protobuf.message.Message):
    """Params defines the parameters for the x/pricefeed module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIRS_FIELD_NUMBER: builtins.int
    TWAP_LOOKBACK_WINDOW_FIELD_NUMBER: builtins.int
    @property
    def pairs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common.common_pb2.AssetPair]:
        """Pairs is the list of valid trading pairs for the module. 
        Add new trading pairs
        """
        pass
    @property
    def twap_lookback_window(self) -> google.protobuf.duration_pb2.Duration:
        """amount of time to look back for TWAP calculations"""
        pass
    def __init__(self,
        *,
        pairs: typing.Optional[typing.Iterable[common.common_pb2.AssetPair]] = ...,
        twap_lookback_window: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["twap_lookback_window",b"twap_lookback_window"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pairs",b"pairs","twap_lookback_window",b"twap_lookback_window"]) -> None: ...
global___Params = Params

class OraclesMarshaler(google.protobuf.message.Message):
    """OraclesMarshaler is a codec.ProtoMarshaler for an oracles array in the 
    OraclesState sdk.KVStore.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ORACLES_FIELD_NUMBER: builtins.int
    @property
    def oracles(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self,
        *,
        oracles: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["oracles",b"oracles"]) -> None: ...
global___OraclesMarshaler = OraclesMarshaler

class ActivePairMarshaler(google.protobuf.message.Message):
    """ActivePairMarshaler is a codec.ProtoMarshaler for the "IsActive" status of  
    a pair in the ActivePairState sdk.KVStore.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    IS_ACTIVE_FIELD_NUMBER: builtins.int
    is_active: builtins.bool
    def __init__(self,
        *,
        is_active: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["is_active",b"is_active"]) -> None: ...
global___ActivePairMarshaler = ActivePairMarshaler

class PostedPrice(google.protobuf.message.Message):
    """PostedPrice defines a price for an asset pair posted by a specific oracle."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_ID_FIELD_NUMBER: builtins.int
    ORACLE_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    EXPIRY_FIELD_NUMBER: builtins.int
    pair_id: typing.Text
    oracle: typing.Text
    price: typing.Text
    @property
    def expiry(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(self,
        *,
        pair_id: typing.Text = ...,
        oracle: typing.Text = ...,
        price: typing.Text = ...,
        expiry: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["expiry",b"expiry"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["expiry",b"expiry","oracle",b"oracle","pair_id",b"pair_id","price",b"price"]) -> None: ...
global___PostedPrice = PostedPrice

class CurrentPrice(google.protobuf.message.Message):
    """CurrentPrice defines the current price for an asset pair in the pricefeed
    module.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_ID_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    pair_id: typing.Text
    price: typing.Text
    def __init__(self,
        *,
        pair_id: typing.Text = ...,
        price: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair_id",b"pair_id","price",b"price"]) -> None: ...
global___CurrentPrice = CurrentPrice

class CurrentTWAP(google.protobuf.message.Message):
    """CurrentTWAP states defines the numerator and denominator for the TWAP calculation"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_ID_FIELD_NUMBER: builtins.int
    NUMERATOR_FIELD_NUMBER: builtins.int
    DENOMINATOR_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    pair_id: typing.Text
    numerator: typing.Text
    denominator: typing.Text
    price: typing.Text
    def __init__(self,
        *,
        pair_id: typing.Text = ...,
        numerator: typing.Text = ...,
        denominator: typing.Text = ...,
        price: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denominator",b"denominator","numerator",b"numerator","pair_id",b"pair_id","price",b"price"]) -> None: ...
global___CurrentTWAP = CurrentTWAP



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import cosmos.staking.v1beta1.staking_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgCreateValidator(google.protobuf.message.Message):
    """MsgCreateValidator defines a SDK message for creating a new validator."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DESCRIPTION_FIELD_NUMBER: builtins.int
    COMMISSION_FIELD_NUMBER: builtins.int
    MIN_SELF_DELEGATION_FIELD_NUMBER: builtins.int
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    PUBKEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    @property
    def description(self) -> cosmos.staking.v1beta1.staking_pb2.Description: ...
    @property
    def commission(self) -> cosmos.staking.v1beta1.staking_pb2.CommissionRates: ...
    min_self_delegation: typing.Text
    delegator_address: typing.Text
    validator_address: typing.Text
    @property
    def pubkey(self) -> google.protobuf.any_pb2.Any: ...
    @property
    def value(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        description: typing.Optional[cosmos.staking.v1beta1.staking_pb2.Description] = ...,
        commission: typing.Optional[cosmos.staking.v1beta1.staking_pb2.CommissionRates] = ...,
        min_self_delegation: typing.Text = ...,
        delegator_address: typing.Text = ...,
        validator_address: typing.Text = ...,
        pubkey: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        value: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["commission",b"commission","description",b"description","pubkey",b"pubkey","value",b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commission",b"commission","delegator_address",b"delegator_address","description",b"description","min_self_delegation",b"min_self_delegation","pubkey",b"pubkey","validator_address",b"validator_address","value",b"value"]) -> None: ...
global___MsgCreateValidator = MsgCreateValidator

class MsgCreateValidatorResponse(google.protobuf.message.Message):
    """MsgCreateValidatorResponse defines the Msg/CreateValidator response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgCreateValidatorResponse = MsgCreateValidatorResponse

class MsgEditValidator(google.protobuf.message.Message):
    """MsgEditValidator defines a SDK message for editing an existing validator."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DESCRIPTION_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    COMMISSION_RATE_FIELD_NUMBER: builtins.int
    MIN_SELF_DELEGATION_FIELD_NUMBER: builtins.int
    @property
    def description(self) -> cosmos.staking.v1beta1.staking_pb2.Description: ...
    validator_address: typing.Text
    commission_rate: typing.Text
    """We pass a reference to the new commission rate and min self delegation as
    it's not mandatory to update. If not updated, the deserialized rate will be
    zero with no way to distinguish if an update was intended.
    REF: #2373
    """

    min_self_delegation: typing.Text
    def __init__(self,
        *,
        description: typing.Optional[cosmos.staking.v1beta1.staking_pb2.Description] = ...,
        validator_address: typing.Text = ...,
        commission_rate: typing.Text = ...,
        min_self_delegation: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["description",b"description"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commission_rate",b"commission_rate","description",b"description","min_self_delegation",b"min_self_delegation","validator_address",b"validator_address"]) -> None: ...
global___MsgEditValidator = MsgEditValidator

class MsgEditValidatorResponse(google.protobuf.message.Message):
    """MsgEditValidatorResponse defines the Msg/EditValidator response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgEditValidatorResponse = MsgEditValidatorResponse

class MsgDelegate(google.protobuf.message.Message):
    """MsgDelegate defines a SDK message for performing a delegation of coins
    from a delegator to a validator.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    validator_address: typing.Text
    @property
    def amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        validator_address: typing.Text = ...,
        amount: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["amount",b"amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","delegator_address",b"delegator_address","validator_address",b"validator_address"]) -> None: ...
global___MsgDelegate = MsgDelegate

class MsgDelegateResponse(google.protobuf.message.Message):
    """MsgDelegateResponse defines the Msg/Delegate response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgDelegateResponse = MsgDelegateResponse

class MsgBeginRedelegate(google.protobuf.message.Message):
    """MsgBeginRedelegate defines a SDK message for performing a redelegation
    of coins from a delegator and source validator to a destination validator.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_SRC_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_DST_ADDRESS_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    validator_src_address: typing.Text
    validator_dst_address: typing.Text
    @property
    def amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        validator_src_address: typing.Text = ...,
        validator_dst_address: typing.Text = ...,
        amount: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["amount",b"amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","delegator_address",b"delegator_address","validator_dst_address",b"validator_dst_address","validator_src_address",b"validator_src_address"]) -> None: ...
global___MsgBeginRedelegate = MsgBeginRedelegate

class MsgBeginRedelegateResponse(google.protobuf.message.Message):
    """MsgBeginRedelegateResponse defines the Msg/BeginRedelegate response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMPLETION_TIME_FIELD_NUMBER: builtins.int
    @property
    def completion_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(self,
        *,
        completion_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["completion_time",b"completion_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["completion_time",b"completion_time"]) -> None: ...
global___MsgBeginRedelegateResponse = MsgBeginRedelegateResponse

class MsgUndelegate(google.protobuf.message.Message):
    """MsgUndelegate defines a SDK message for performing an undelegation from a
    delegate and a validator.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    validator_address: typing.Text
    @property
    def amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        validator_address: typing.Text = ...,
        amount: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["amount",b"amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","delegator_address",b"delegator_address","validator_address",b"validator_address"]) -> None: ...
global___MsgUndelegate = MsgUndelegate

class MsgUndelegateResponse(google.protobuf.message.Message):
    """MsgUndelegateResponse defines the Msg/Undelegate response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMPLETION_TIME_FIELD_NUMBER: builtins.int
    @property
    def completion_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(self,
        *,
        completion_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["completion_time",b"completion_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["completion_time",b"completion_time"]) -> None: ...
global___MsgUndelegateResponse = MsgUndelegateResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.mint.v1beta1.mint_pb2
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is the request type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is the response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> cosmos.mint.v1beta1.mint_pb2.Params:
        """params defines the parameters of the module."""
        pass
    def __init__(self,
        *,
        params: typing.Optional[cosmos.mint.v1beta1.mint_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse

class QueryInflationRequest(google.protobuf.message.Message):
    """QueryInflationRequest is the request type for the Query/Inflation RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryInflationRequest = QueryInflationRequest

class QueryInflationResponse(google.protobuf.message.Message):
    """QueryInflationResponse is the response type for the Query/Inflation RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INFLATION_FIELD_NUMBER: builtins.int
    inflation: builtins.bytes
    """inflation is the current minting inflation value."""

    def __init__(self,
        *,
        inflation: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["inflation",b"inflation"]) -> None: ...
global___QueryInflationResponse = QueryInflationResponse

class QueryAnnualProvisionsRequest(google.protobuf.message.Message):
    """QueryAnnualProvisionsRequest is the request type for the
    Query/AnnualProvisions RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryAnnualProvisionsRequest = QueryAnnualProvisionsRequest

class QueryAnnualProvisionsResponse(google.protobuf.message.Message):
    """QueryAnnualProvisionsResponse is the response type for the
    Query/AnnualProvisions RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ANNUAL_PROVISIONS_FIELD_NUMBER: builtins.int
    annual_provisions: builtins.bytes
    """annual_provisions is the current minting annual provisions value."""

    def __init__(self,
        *,
        annual_provisions: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["annual_provisions",b"annual_provisions"]) -> None: ...
global___QueryAnnualProvisionsResponse = QueryAnnualProvisionsResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Pairs(google.protobuf.message.Message):
    """Pairs defines a repeated slice of Pair objects."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIRS_FIELD_NUMBER: builtins.int
    @property
    def pairs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Pair]: ...
    def __init__(self,
        *,
        pairs: typing.Optional[typing.Iterable[global___Pair]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pairs",b"pairs"]) -> None: ...
global___Pairs = Pairs

class Pair(google.protobuf.message.Message):
    """Pair defines a key/value bytes tuple."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    def __init__(self,
        *,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...
global___Pair = Pair



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SendAuthorization(google.protobuf.message.Message):
    """SendAuthorization allows the grantee to spend up to spend_limit coins from
    the granter's account.

    Since: cosmos-sdk 0.43
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SPEND_LIMIT_FIELD_NUMBER: builtins.int
    @property
    def spend_limit(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        spend_limit: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["spend_limit",b"spend_limit"]) -> None: ...
global___SendAuthorization = SendAuthorization



from datetime import datetime

from nibiru import utils
from nibiru.proto.pricefeed import tx_pb2 as pb


class Pricefeed:
    @staticmethod
    def post_price(oracle: str, token0: str, token1: str, price: float, expiry: datetime):
        price_dec = utils.to_sdk_dec(price)
        return pb.MsgPostPrice(
            oracle=oracle,
            token0=token0,
            token1=token1,
            price=price_dec,
            expiry=utils.toTsPb(expiry),
        )
